divert(-1)

#
# This file contains the OpenSIPS settings and routing logic.
#
# This file should NOT be modified. Changes made to this file will NOT be
# preserved over upgrades.
#
# To change the proxy behavior, there is the settings.m4 file which contains
# parameters that can be modified to customize the installation.
# If the basic options available in settings.m4 are not enough to satisfy the
# specific needs of an installation, there are a number of site specific files
# under the siteconfig subdirectory, in which specific routing logic can be
# added to satisfy those needs.
# Changes made to settings.m4 and the site specific files under siteconfig
# will never be overwritten.
#
# After modifying settings.m4 and the site specific files, the opensips.cfg
# file can be regenerated by running:
#
#     opensips-config
#
# which will also check if the new configuration is free of errors.
#

################
# Basic settings
################

# Unsafe but a good dtart to see your proxy working
define(`ENABLE_AUTHENTICATION', `1')

# relay RTP packets using mediaproxy (necessary to receive calls behind NAT)
define(`ENABLE_MEDIAPROXY', `0')

# The IP on which OpenSIPS will listen (by default the IP for the default interface)
define(`SERVER_IP', esyscmd(`echo -n $(hostname -i)'))

# MySQL parameters
define(`MySQL_URI', `mysql://opensips:opensips@localhost/opensips')

# TLS advanced parameters
define(`TLS_VERIFY_CERTIFICATE', `0')

###################
# Advanced settings
###################

define(`MySQL_URI_location', `MySQL_URI')
define(`MySQL_URI_domain',   `MySQL_URI')
define(`MySQL_URI_siptrace', `MySQL_URI')

# SERVER aliases
# Example:
#  define(`SERVER_ALIAS1', `example.com')
#  define(`SERVER_ALIAS2', `example.net')
#  define(`SERVER_ALIASES', `2')
define(`SERVER_ALIASES', `0')

# How many worker processes to spawn (will start with min but will grow up to
# max as load increases; will also scale down back to min if load decreases)
# TCP and UDP worker pools are separate and vary independently
define(`MIN_WORKERS', `5')
define(`MAX_WORKERS', `25')

# Enable dialog pinging
define(`ENABLE_DIALOG_PING', `0')

# Dialog ping interval
define(`DIALOG_PING_INTERVAL', `30')

# Set this to 1 for encrypted passwords or 0 for plaintext passwords
# Boolean value (0 or 1 only)
define(`ENCRYPTED_PASSWORDS', `1')


# Quality of protection (qop) that will be used with digest authentication.
# It can be one of 'auth', 'auth-int', 'auth,auth-int' or '' (w/o quotes).
# It is recommended to use qop for enhanced security, but some devices might
# not support it, in which case it can be disabled by setting it to be empty.
define(`QUALITY_OF_PROTECTION', `auth,auth-int')

# Comment the prompts below that you do not want to use

# The log levels defined below can have one of these values:
#
#   -1: ERROR, 1: WARNING, 2: NOTICE, 3: INFO, 4: DEBUG
#
# The log levels can be adjusted at runtime using the MI interface.

# The log level for the opensips processes
define(`PROC_LOG_LEVEL', `3')

# The log level for the configuration script
define(`CONF_LOG_LEVEL', `3')

# Enable configuration logging for REGISTER. Boolean value (0 or 1 only)
define(`REGISTER_LOGGING', `0')

# How many open files can opensips process (this affects how many TCP/TLS
# connections can be accepted). Increasing this increases memory usage and
# reduces performance. Valid values are in the 1000-1000000 range.
define(`LIMIT_FILENO', `1000')

# MySQL advanced parameters
define(`DOMAIN_TABLE',     `domain')
define(`ALIASES_TABLE',    `dbaliases')
define(`LOCATION_TABLE',   `location')
define(`SUBSCRIBER_TABLE', `subscriber')

# Timeouts (INVITE/other methods)
define(`INVITE_TIMEOUT',   `35')
define(`OTHERS_TIMEOUT',   `15')
define(`OUTGOING_TIMEOUT', `120')

# Registration expiration restrictions
# DEFAULT_EXPIRE is for clients that do not set an expiration time
# MIN and MAX_EXPIRE will force expiration times to be contained in that
# interval, no matter what the client proposes.
define(`MIN_EXPIRE',     `60')
define(`MAX_EXPIRE',     `3600')
define(`DEFAULT_EXPIRE', `300')

# Interval to ping the clients behind NAT to keep the pinholes open
define(`NAT_KEEPALIVE_INTERVAL', `80')

# QoS control. The type of service (TOS) value can be used to control the
# traffic priority at the IP level, however it has to be supported by the
# switches and routers in the path to actually work. The value is either a
# hex value written like 0x10, or one of the following keywords:
#   IPTOS_LOWDELAY, IPTOS_THROUGHPUT, IPTOS_RELIABILITY or IPTOS_MINCOST
define(`TOS', `IPTOS_LOWDELAY')

# MediaProxy ICE candidate (none | low-priority | high-priority)
define(`MEDIAPROXY_ICE_CANDIDATE', `none')

define(`SERVER_UDP_PORT', `5060')
define(`SERVER_TCP_PORT', `5060')
define(`SERVER_TLS_PORT', `5061')

define(`SERVER_TCP_SOCKET', `tcp:SERVER_IP:SERVER_TCP_PORT')
define(`SERVER_UDP_SOCKET', `udp:SERVER_IP:SERVER_UDP_PORT')
define(`SERVER_TLS_SOCKET', `tls:SERVER_IP:SERVER_TLS_PORT')

define(`ENABLE_SYLK_MESSAGE_STORAGE', `0')
define(`CONFERENCE_SERVER_PREFIX', `conference')
define(`SYLK_SERVER_IP', `127.0.0.1')
define(`SYLK_SERVER_PORT', `5060')
define(`SYLK_SERVER_TRANSPORT', `tcp')
define(`SYLK_SERVER_SOCKET', `SERVER_TCP_SOCKET')

# Set to 1 to enable tracing, or 0 to disable it
# Boolean value (0 or 1 only)
define(`TRACING', `0')

#
# End of definitions section
#
define(`MAX_REGISTERED_CONTACTS', `10')

# The secret used to compute the nonce for authorizations. In a SIP Thor
# network, the secret must be the same on all the network members to avoid
# issues with registrations failing when a subscriber is moved to a different
# node while the network topology changes. In such a case the UA may receive
# two 401 in a row if it happens to register while a network change involving
# the subscribe home node occurs. If this happens the UA will give up on the
# registartion attempt and ask the user for the password. Same for clusters.
define(`AUTH_SECRET', `makeithere')

# OpenSIPS modules directory
define(`MODULES_DIR', esyscmd(`echo -n /usr/lib/$(dpkg-architecture -qDEB_HOST_MULTIARCH)/opensips/modules'))

# OpenSIPS configuration directory
define(`OPENSIPS_DIR',                  `/etc/opensips')
define(`CONFIG_DIR',                    `OPENSIPS_DIR/config')
# Paths to the optional files that can alter routing logic if present
define(`SETTINGS_M4',                   `CONFIG_DIR/settings.m4')
define(`PREPROCESS_REQUEST_M4',         `CONFIG_DIR/siteconfig/preprocess-request.m4')
define(`PREPROCESS_BRANCH_M4',          `CONFIG_DIR/siteconfig/preprocess-branch.m4')
define(`POSTPROCESS_REQUEST_M4',        `CONFIG_DIR/siteconfig/postprocess-request.m4')
define(`POSTPROCESS_REGISTER_M4',       `CONFIG_DIR/siteconfig/postprocess-register.m4')
define(`HANDLE_USER_NOT_FOUND_M4',      `CONFIG_DIR/siteconfig/handle-user-not-found.m4')
define(`PREPROCESS_MESSAGE_M4',         `CONFIG_DIR/siteconfig/preprocess-message.m4')
define(`HANDLE_POST_AUTHENTICATION_M4', `CONFIG_DIR/siteconfig/handle-post-authentication.m4')

# Include configuration specific to this site if available
# (this will overwrite variables defined until here)
sinclude(SETTINGS_M4)

#
# Internals
#

# OpenSIPS version
define(`__VERSION__', `3.1')

# The server addresses and contact URIs
define(`SERVER_TLS_ADDRESS', `ifelse(SERVER_TLS_PORT, `5061', `SERVER_IP', `SERVER_IP:SERVER_TLS_PORT')')
define(`SERVER_TCP_ADDRESS', `ifelse(SERVER_UDP_PORT, `5060', `SERVER_IP', `SERVER_IP:SERVER_TCP_PORT')')
define(`SERVER_UDP_ADDRESS', `ifelse(SERVER_UDP_PORT, `5060', `SERVER_IP', `SERVER_IP:SERVER_UDP_PORT')')

define(`SERVER_TLS_URI', `sip:SERVER_TLS_ADDRESS;transport=tls')
define(`SERVER_TCP_URI', `sip:SERVER_TCP_ADDRESS;transport=tcp')
define(`SERVER_UDP_URI', `sip:SERVER_UDP_ADDRESS')

# The server identification string
define(`SERVER_IDENTIFICATION', `"Server: OpenSIPS AG __VERSION__"')')

define(`UAC_IDENTIFICATION', `"User-Agent: OpenSIPS AG __VERSION__"')')

# The relay timeout should be much longer than any other timeout and it should
# be used by any proxy not responsible for the destination account that is
# forwarding the message (NAT entry points, thor relays, on diversions).
define(`RELAY_TIMEOUT',  `min(300, eval(OUTGOING_TIMEOUT * 5))')


#
# Macro section
#

# print an error message and exit
define(`fatal_error', `errprint(`m4: '__file__:__line__`: fatal error: $*
')m4exit(1)')

# min/max of a list of numbers
define(`_min2', `ifelse(eval($1 < $2), `1', $1, $2)')
define(`_max2', `ifelse(eval($1 > $2), `1', $1, $2)')
define(`min',   `ifelse($#, 0, , $#, 1, `$1', `_min2($1, min(shift($@)))')')
define(`max',   `ifelse($#, 0, , $#, 1, `$1', `_max2($1, max(shift($@)))')')

# for loop
define(`forloop',
       `pushdef(`$1', `$2')_forloop(`$1', `$2', `$3', `$4')popdef(`$1')')
define(`_forloop',
       `$4`'ifelse($1, `$3', ,
           `define(`$1', incr($1))_forloop(`$1', `$2', `$3', `$4')')')

# if/else for M4 (to use instead of the ifelse from M4 which is hard to read)
# this if/else construct cannot be nested except when the interior if/else
# structure is right before the parent`s M4_ELSE or M4_ENDIF element!!!
define(`M4_IF',    `ifelse(eval(`$#' < `2'), `1', `_m4_if1($*)', `_m4_if2($*)')')
define(`M4_ELSE',  `ifelse(eval(`$#' < `2'), `1', `_m4_else1($*)', `_m4_else2($*)')')
define(`M4_ENDIF', `divert(0)dnl')
define(`M4_IFDEF', `ifdef(`$1', `divert(0)dnl', `divert(-1)dnl')')

# if/else helpers
define(`_m4_if1',   `ifelse(`$1', `0', `divert(-1)dnl', `divert(0)dnl')')
define(`_m4_else1', `ifelse(`$1', `0', `divert(0)dnl', `divert(-1)dnl')')
define(`_m4_if2',   `ifelse(`$1', `$2', `divert(0)dnl', `divert(-1)dnl')')
define(`_m4_else2', `ifelse(`$1', `$2', `divert(-1)dnl', `divert(0)dnl')')


# Logging macros
define(`LOG_PREFIX', `[CONFIG] ')

define(`INFO', `xlog("L_INFO", patsubst(`$*', `^"\(.*\)"$', `"LOG_PREFIX`'\1\\n"'))')
define(`WARN', `xlog("L_WARN", patsubst(`$*', `^"\(.*\)"$', `"LOG_PREFIX`WARNING: '\1\\n"'))')
define(`ERROR', `xlog("L_ERR", patsubst(`$*', `^"\(.*\)"$', `"LOG_PREFIX`ERROR: '\1\\n"'))')
define(`DEBUG', `xlog("L_DBG", patsubst(`$*', `^"\(.*\)"$', `"LOG_PREFIX`'\1\\n"'))')

define(`RINFO', ifelse(REGISTER_LOGGING, `1', ``INFO($@)'', `'))dnl
define(`RWARN', ifelse(REGISTER_LOGGING, `1', ``WARN($@)'', `'))dnl
define(`RERROR', ifelse(REGISTER_LOGGING, `1', ``ERROR($@)'', `'))dnl
define(`RDEBUG', ifelse(REGISTER_LOGGING, `1', ``DEBUG($@)'', `'))dnl

# Test condition macros
define(`IS_REGISTER',      `$rm=="REGISTER"')dnl
define(`IS_INVITE',        `$rm=="INVITE"')dnl
define(`IS_ACK',           `$rm=="ACK"')dnl
define(`IS_PRACK',         `$rm=="PRACK"')dnl
define(`IS_BYE',           `$rm=="BYE"')dnl
define(`IS_CANCEL',        `$rm=="CANCEL"')dnl
define(`IS_UPDATE',        `$rm=="UPDATE"')dnl
define(`IS_MESSAGE',       `$rm=="MESSAGE"')dnl
define(`IS_NOTIFY',        `$rm=="NOTIFY"')dnl
define(`IS_SUBSCRIBE',     `$rm=="SUBSCRIBE"')dnl
define(`IS_OPTIONS',       `$rm=="OPTIONS"')dnl
define(`IS_REFER',         `$rm=="REFER"')dnl
define(`IS_PUBLISH',       `$rm=="PUBLISH"')dnl
define(`IS_PING',          `$rm=="PING"')dnl
define(`IS_INFO',          ``$rm=="INFO"'')dnl
define(`IS_NOT_REGISTER',  `$rm!="REGISTER"')dnl
define(`IS_NOT_ACK',       `$rm!="ACK"')dnl
define(`IS_NOT_PRACK',     `$rm!="PRACK"')dnl
define(`IS_NOT_CANCEL',    `$rm!="CANCEL"')dnl
define(`IS_NOT_UPDATE',    `$rm!="UPDATE"')dnl
define(`IS_NOT_MESSAGE',   `$rm!="MESSAGE"')dnl
define(`IS_NOT_NOTIFY',    `$rm!="NOTIFY"')dnl
define(`IS_NOT_SUBSCRIBE', `$rm!="SUBSCRIBE"')dnl
define(`IS_NOT_OPTIONS',   `$rm!="OPTIONS"')dnl
define(`IS_NOT_REFER',     `$rm!="REFER"')dnl
define(`IS_NOT_PUBLISH',   `$rm!="PUBLISH"')dnl
define(`IS_NOT_PING',      `$rm!="PING"')dnl
define(`IS_NOT_INFO',      ``$rm!="INFO"'')dnl

define(`IN_DIALOG',        `has_totag()')dnl
define(`OUT_OF_DIALOG',    `not has_totag()')dnl

define(`IS_FIRST_INVITE',  `(IS_INVITE and OUT_OF_DIALOG)')dnl
define(`CALL_METHODS',     `(IS_INVITE or IS_CANCEL or IS_BYE or IS_ACK or IS_PRACK or IS_UPDATE or IS_INFO)')dnl

define(`ONE_LEG_IS_LOCAL',    `(is_from_local() or is_uri_host_local())')dnl
define(`BOTH_LEGS_ARE_LOCAL', `(is_from_local() and is_uri_host_local())')dnl

define(`LOCAL_TIMEOUT', `(t_check_status("408") and t_local_replied("last"))')dnl


# Flag tests
#

# Transaction flags
define(`DO_AUTHENTICATE',        `isflagset("DO_AUTHENTICATE_FLAG")')dnl
define(`FROM_SELF',              `isflagset("FROM_SELF_FLAG")')dnl
define(`BEHIND_NAT',             `isflagset("NAT_FLAG")')dnl

# Flag test aliases (better meaning in different contexts)
define(`DID_AUTHENTICATE',       `DO_AUTHENTICATE')dnl
define(`DID_LOAD_PROFILE',       `DO_LOAD_PROFILE')dnl

define(`_divert', ``divert'($1)')dnl

define(`www_challenge',   ifelse(defn(`QUALITY_OF_PROTECTION'), `', ```www_challenge'($1)'',   ```www_challenge'($1, "QUALITY_OF_PROTECTION")''))dnl
define(`proxy_challenge', ifelse(defn(`QUALITY_OF_PROTECTION'), `', ```proxy_challenge'($1)'', ```proxy_challenge'($1, "QUALITY_OF_PROTECTION")''))dnl

# Numeric value for $1 must be bigger than numeric value for $2, else exit.
define(`must_be_bigger', `ifelse(eval($1 > $2), `0', `fatal_error(`$1 should be bigger than $2')')')


#
# Checks on the configuration go here. Everything we need to check for
# consistency before we generate the configuration
#
must_be_bigger(`INVITE_TIMEOUT',   `OTHERS_TIMEOUT')
must_be_bigger(`OUTGOING_TIMEOUT', `INVITE_TIMEOUT')


divert(0)dnl
#
# This is a generated file. Changes made to this file will be overwritten when
# this file is regenerated. Please edit config/settings.m4 and the files under
# the config/siteconfig subdirectory to alter the proxy routing behavior.
#
# After changing the appropriate m4 files run:
#
#     opensips-config
#
# to regenerate the configuration file.
#

# Process options
#
debug_mode   = no
log_stderror = no
log_facility = LOG_LOCAL0
log_level    = PROC_LOG_LEVEL
xlog_level   = CONF_LOG_LEVEL

open_files_limit = LIMIT_FILENO

# Server settings
#
socket = udp:SERVER_IP:SERVER_UDP_PORT
socket = tcp:SERVER_IP:SERVER_TCP_PORT
socket = tls:SERVER_IP:SERVER_TLS_PORT

socket = udp:127.0.0.1:SERVER_UDP_PORT
socket = tcp:127.0.0.1:SERVER_TCP_PORT
socket = tls:127.0.0.1:SERVER_TLS_PORT


auto_scaling_profile = SIP_WORKERS
     scale up to MAX_WORKERS on 80% for 4 cycles within 5
     scale down to MIN_WORKERS on 20% for 10 cycles

tcp_workers = MIN_WORKERS use_auto_scaling_profile SIP_WORKERS
udp_workers = MIN_WORKERS use_auto_scaling_profile SIP_WORKERS

tos = TOS

# TCP/IP settings
#
tcp_accept_aliases      = no
tcp_max_connections     = LIMIT_FILENO
tcp_no_new_conn_bflag   = TCP_NO_NEW_CONN_BFLAG
tcp_connection_lifetime = 3600  # seconds
tcp_connect_timeout     = 2000   # milliseconds
tcp_keepalive           = yes
tcp_keepcount           = 4
tcp_keepidle            = 60
tcp_keepinterval        = 20

# SIP settings
sip_warning   = 0
check_via     = no
server_header = SERVER_IDENTIFICATION
user_agent_header = UAC_IDENTIFICATION

# DNS settings
dns                   = yes
rev_dns               = no
dns_use_search_list   = no
disable_dns_blacklist = yes

# Default DB for all modules
db_default_url="MySQL_URI"

# Modules
#
mpath = "MODULES_DIR"
loadmodule "proto_udp.so"
loadmodule "proto_tcp.so"
loadmodule "proto_tls.so"
loadmodule "tls_mgm.so"
loadmodule "mi_fifo.so"
loadmodule "mi_datagram.so"
loadmodule "db_mysql.so"
loadmodule "maxfwd.so"
loadmodule "sl.so"
loadmodule "tm.so"
loadmodule "signaling.so"
loadmodule "rr.so"
loadmodule "usrloc.so"
loadmodule "registrar.so"
loadmodule "auth.so"
loadmodule "auth_db.so"
loadmodule "cfgutils.so"
loadmodule "exec.so"
loadmodule "sipmsgops.so"
loadmodule "textops.so"
loadmodule "domain.so"
loadmodule "options.so"
loadmodule "avpops.so"
loadmodule "permissions.so"
loadmodule "dialog.so"
M4_IF(ENABLE_MEDIAPROXY)
loadmodule "mediaproxy.so"
M4_ENDIF(ENABLE_MEDIAPROXY)
loadmodule "nat_traversal.so"
loadmodule "tracer.so"

# Module parameters
#

# Protocol parameters
#
modparam("proto_tcp", "tcp_async", 1)
modparam("proto_tcp", "tcp_max_msg_chunks", 50)
modparam("proto_tcp", "tcp_send_timeout", 2000)       # milliseconds

modparam("proto_tls", "tls_max_msg_chunks", 50)
modparam("proto_tls", "tls_send_timeout", 2000)       # milliseconds
modparam("proto_tls", "tls_handshake_timeout", 5000)  # milliseconds - must be at least twice tcp_connect_timeout plus some extra 10-20ms

# TLS domain settings
modparam("tls_mgm", "server_domain",    "server-default")
modparam("tls_mgm", "match_ip_address", "[server-default]*")
modparam("tls_mgm", "match_sip_domain", "[server-default]*")
modparam("tls_mgm", "tls_method",       "[server-default]TLSv1-")
modparam("tls_mgm", "certificate",      "[server-default]/etc/opensips/tls/default.crt")
modparam("tls_mgm", "private_key",      "[server-default]/etc/opensips/tls/default.key")
modparam("tls_mgm", "ca_list",          "[server-default]/etc/opensips/tls/ca-list.pem")
modparam("tls_mgm", "ca_dir",           "[server-default]/etc/ssl/certs")
modparam("tls_mgm", "verify_cert",      "[server-default]0")
modparam("tls_mgm", "require_cert",     "[server-default]0")

modparam("tls_mgm", "client_domain",    "client-default")
modparam("tls_mgm", "match_ip_address", "[client-default]*")
modparam("tls_mgm", "match_sip_domain", "[client-default]*")
modparam("tls_mgm", "tls_method",       "[client-default]TLSv1-")
modparam("tls_mgm", "certificate",      "[client-default]/etc/opensips/tls/default.crt")
modparam("tls_mgm", "private_key",      "[client-default]/etc/opensips/tls/default.key")
modparam("tls_mgm", "ca_list",          "[client-default]/etc/opensips/tls/ca-list.pem")
modparam("tls_mgm", "ca_dir",           "[client-default]/etc/ssl/certs")
modparam("tls_mgm", "verify_cert",      "[client-default]0")
modparam("tls_mgm", "require_cert",     "[client-default]0")

# parameters set for multiple modules at once
modparam("auth_db|avpops|usrloc", "use_domain", 1)

modparam("auth", "secret", "AUTH_SECRET")

M4_IF(ENCRYPTED_PASSWORDS)
modparam("auth_db", "calculate_ha1",     0)
modparam("auth_db", "password_column",   "ha1")
modparam("auth_db", "password_column_2", "ha1b")
M4_ELSE(ENCRYPTED_PASSWORDS)
modparam("auth_db", "calculate_ha1",     1)
modparam("auth_db", "password_column",   "password")
M4_ENDIF(ENCRYPTED_PASSWORDS)
modparam("auth_db", "user_column",       "username")
modparam("auth_db", "domain_column",     "domain")
modparam("auth_db", "load_credentials",  "")

modparam("domain", "db_url", "MySQL_URI_domain")
modparam("domain", "db_mode", 1)
modparam("domain", "domain_table", "DOMAIN_TABLE")

modparam("mi_datagram", "socket_name", "/var/run/opensips/socket")
modparam("mi_datagram", "unix_socket_user",  "opensips")
modparam("mi_datagram", "unix_socket_group", "opensips")

modparam("mi_fifo", "fifo_name", "/var/run/opensips/fifo")
modparam("mi_fifo", "fifo_user",  "opensips")
modparam("mi_fifo", "fifo_group", "opensips")

modparam("nat_traversal", "keepalive_interval", NAT_KEEPALIVE_INTERVAL)

modparam("registrar", "case_sensitive", 0)
modparam("registrar", "max_contacts", MAX_REGISTERED_CONTACTS)
modparam("registrar", "retry_after", MIN_EXPIRE)
modparam("registrar", "min_expires", MIN_EXPIRE)
modparam("registrar", "max_expires", MAX_EXPIRE)
modparam("registrar", "default_expires", DEFAULT_EXPIRE)
modparam("registrar", "tcp_persistent_flag", "PERSISTENT_TCP_FLAG")
modparam("registrar", "received_avp", "$avp(received_uri)")

# How long will OpenSIPS wait for a final response
# fr_inv_timeout sets value for INVITE transactions, fr_timeout for all others
modparam("tm", "fr_inv_timeout", INVITE_TIMEOUT)
modparam("tm", "fr_timeout", OTHERS_TIMEOUT)
modparam("tm", "restart_fr_on_each_reply", 0)

modparam("usrloc", "db_url", "MySQL_URI_location")
modparam("usrloc", "working_mode_preset", "single-instance-sql-write-through")
modparam("usrloc", "desc_time_order", 1)

M4_IF(ENABLE_MEDIAPROXY)
modparam("dialog", "dlg_match_mode", 1)
modparam("dialog", "options_ping_interval", DIALOG_PING_INTERVAL)
modparam("mediaproxy", "ice_candidate", "MEDIAPROXY_ICE_CANDIDATE")

M4_ENDIF(ENABLE_MEDIAPROXY)

modparam("tracer", "trace_on", TRACING)
modparam("tracer", "trace_id", "[default] uri=MySQL_URI_siptrace; table=sip_trace")

# -------------------------  routing logic  -------------------------

#
# main route
#
route {

    # Message checks: relaying, max forwards, ...
    
    if (not mf_process_maxfwd_header(10)) {
        WARN("Too many hops for $rm from $fu to $ru via $si:$sp");
        if (IS_NOT_ACK) {
            force_rport();
            sl_send_reply(483, "Too many hops");
        }
        return;
    }

    if (is_myself($si) and is_myself($(hdr(Via){via.host}), $(hdr(Via){via.port}))) {
        setflag("FROM_SELF_FLAG");
    }

    # Do NOT use the FROM_SELF test before this point as it will be undefined

    # Handle keepalive (ping) messages from clients

    if (IS_PING or (IS_NOTIFY and avp_check("$hdr(Event)", "eq/keep-alive/i"))) {
        force_rport();
        sl_send_reply(200, "OK");
        return;
    }

    if (IS_OPTIONS and $ru !~ "sip:.*@.*") {
        force_rport();
        options_reply();
        return;
    }

    if (not FROM_SELF and OUT_OF_DIALOG and IS_NOT_REGISTER) {
        if (not (FROM_SELF or ONE_LEG_IS_LOCAL)) {
            DEBUG("$rm $ru from $fu to $tu via $socket_in(proto):$si:$sp ($ci) - relaying denied");
            force_rport();
            sl_send_reply(403, "Relaying denied");
            return;
        }
    }

    if (FROM_SELF) {
        $var(source_id) = "self";
    } else if (is_from_local()) {
        $var(source_id) = "local user";
    } else {
        $var(source_id) = "extern";
        remove_hf("X-Sylk-From-Sip");
        remove_hf("X-Sylk-App");
        remove_hf("X-Replicated-Message");
    }

    if (IS_REGISTER) {
        DEBUG("$rm $tu with $(ct{s.select,0,>})> via $var(source_id) $socket_in(proto):$si:$sp");
    } else {
        if (CALL_METHODS) {
            if ($ru != $tu) { 
                INFO("$rm $ru CALL from $fu to $tu via $var(source_id) $socket_in(proto):$si:$sp ($ci)");
            } else {
                INFO("$rm CALL from $fu to $tu via $var(source_id) $socket_in(proto):$si:$sp ($ci)");
            }
        } else if (IS_MESSAGE){
            INFO("$rm $hdr(Content-Type) from $fu to $tu via $var(source_id) $socket_in(proto):$si:$sp ($ci)");
        } else {
            DEBUG("$rm $ru from $fu to $tu via $var(source_id) $socket_in(proto):$si:$sp ($ci)");
        }
    }

    if (IS_INVITE and $socket_in(proto) == "TLS" and $tls_peer_subject_cn and $tls_peer_verified) {
        INFO("SIP TLS CALL from $si:$sp CN=$tls_peer_subject_cn (verified) $fu -> $ru");
        $avp(tls_cn) = $tls_peer_subject_cn;
    } else {
        $avp(tls_cn) = "";
    }
    
    $avp(sip_proxy_ip) = "SERVER_IP";
    $avp(source_ip)   = $si;
    $avp(source_port) = $sp;

    sinclude(PREPROCESS_REQUEST_M4)dnl;

    # Handle registration

    if (IS_REGISTER) {
        route(REGISTER);
        return;
    }

    # Handle everything else

    if (not FROM_SELF and client_nat_test(3) and not is_present_hf("Record-Route")) {
        # Mark as NATed
        setflag("NAT_FLAG");
        force_rport();
        if (is_present_hf("Contact")) {
            if ($(ct.fields(uri){uri.params}) != "" and $(ct.fields(uri){uri.params}{param.exist,gr}) == 1) {
                # Contact is a GRUU, do not fix it
                DEBUG("Not fixing GRUU contact");
            } else {
                fix_contact();
            }
        }
    }

    if (IS_FIRST_INVITE) {
        trace("default", "d", "sip");

        if (loose_route()) {
            WARN("Incorrectly formatted $rm request. Rejected with 400. ($ci)");
            sl_send_reply(400, "Incorrectly formatted request");
            return;
        }

        # Stop retransmissions
        t_check_trans();

        if (has_body("application/sdp")) {
            if (search_body("m=audio")) {
                $avp(sip_application_type) = "audio";
            }
            if (search_body("m=video")) {
                if (is_avp_set("$avp(sip_application_type)")) {
                    pv_printf($avp(sip_application_type), "$avp(sip_application_type), video");
                } else {
                    $avp(sip_application_type) = "video";
                }
            }
            if (search_body("(m=message).*?(MSRP)")) {
                if (search_body("a=file-selector")) {
                    if (is_avp_set("$avp(sip_application_type)")) {
                        pv_printf($avp(sip_application_type), "$avp(sip_application_type), file-transfer");
                    } else {
                        $avp(sip_application_type) = "file-transfer";
                    }
                } else {
                    if (is_avp_set("$avp(sip_application_type)")) {
                        pv_printf($avp(sip_application_type), "$avp(sip_application_type), chat");
                    } else {
                        $avp(sip_application_type) = "chat";
                    }
                }
            }
        } else {
            $avp(sip_application_type) = "audio";
        }

        INFO("$rm $ru $avp(sip_application_type) CALL from $fu to $tu via $var(source_id) $socket_in(proto):$si:$sp ($ci)");


        if (FROM_SELF) {
            INFO("Request came from self ($ci)");
            if (not is_present_hf("Self-Forwarded")) {
                WARN("Detected call to non-local domain that resolves to this proxy in DNS!");
                INFO("$rm $ru CALL from $fu to $tu via $var(source_id) $socket_in(proto):$si:$sp ($ci)");
                sl_send_reply(404, "Domain not served here");
                return;
            }

            setflag("DIVERTED_CALL_FLAG");

            sl_send_reply(100, "Trying");
        } else if (is_from_local()) {
M4_IF(ENABLE_AUTHENTICATION)
            setflag("DO_AUTHENTICATE_FLAG");
M4_ENDIF(ENABLE_AUTHENTICATION)

            if (BEHIND_NAT) {
                nat_keepalive();
            }

            if (DO_AUTHENTICATE) {
                $var(local_user) = "sip:" + $fU + "@" + $fd;
                if (not db_does_uri_exist($var(local_user), "subscriber")) {
                     sinclude(HANDLE_USER_NOT_FOUND_M4)dnl;
                     sl_send_reply(404, "User not found");
                     return;
                }

                if (not proxy_authorize("", "SUBSCRIBER_TABLE")) {
                    INFO("Asking $fU@$fd for credentials ($ci)");
                    proxy_challenge("");
                    return;
                } else if ($au != $fU) {
                    WARN("Username!=From not allowed ($au!=$fU) ($ci)");
                    sl_send_reply(403, "Username!=From not allowed");
                    return;
                }

                if ($rd == "") {
                    $rd = $fd;
                }

                DEBUG("Caller fullname: $avp(first_name) $avp(last_name)");

                # Hide auth credentials before sending out
                consume_credentials();

            }

            #
            # 100 Trying is send only after we have authenticated the user,
            # so that unauthenticated INVITEs will not receive 100 before 407
            #
            # There is still a chance that if the database is locked or
            # takes more than 1 second to answer, that we get multiple
            # INVITEs which will be handled separately instead of being
            # considered retransmisions and can generate different answers.
            # However if 100 is sent before authentication, stateless
            # replies like 407 may be lost and never retransmitted and the
            # call may not be able to setup.
            #
            # Either way there is a different issue that must be accepted.
            #
            sl_send_reply(100, "Trying");

            if (DID_AUTHENTICATE) {
                sinclude(HANDLE_POST_AUTHENTICATION_M4)dnl;
            }
        } else {
            #
            sl_send_reply(100, "Trying");

        }

        route(LOOKUP_DESTINATION);

        t_on_reply("INVITE_REPLY");
        t_on_failure("FAILED_CALL");
    } else if (IS_MESSAGE) {
        if (db_does_uri_exist($ru, "subscriber")) {
            t_on_failure("FAILED_MESSAGE");
        }

        $avp(sip_application_type) = "message";
        $avp(can_uri) = $tu;
        $avp(source_ip) = $si;
        $avp(source_port) = $sp;
        $avp(sip_proxy_ip) = "SERVER_IP";

        if (OUT_OF_DIALOG) {
            t_on_reply("DIALOG_REPLY");
            if (loose_route()) {
                WARN("Incorrectly formatted $rm request. Rejected with 400. ($ci)");
                sl_send_reply(400, "Incorrectly formatted request");
                return;
            }

            if (is_from_local()) {
M4_IF(ENABLE_AUTHENTICATION)
                if (not FROM_SELF) {
                    setflag("DO_AUTHENTICATE_FLAG");
                }
M4_ENDIF(ENABLE_AUTHENTICATION)
                if (DO_AUTHENTICATE) {
                    if (not proxy_authorize("", "SUBSCRIBER_TABLE")) {
                        INFO("Asking for $fU@$fd credentials ($ci)");
                        proxy_challenge("");
                        return;
                    } else if ($au != $fU) {
                        INFO("Rejected with 403 because $au != $fU ($ci)");
                        sl_send_reply(403, "Username!=From not allowed ($au!=$fU)");
                        return;
                    }

                    if ($rd == "") {
                        $rd = $fd;
                    }

                    # Hide auth credentials to downstream routers
                    consume_credentials();
                }

                # if should for message to ourselves to have a copy on other online devices 
                $var(fork_to_myself) = 1;

                $var(orig_ru) = $ru;
                $var(orig_ruser) = $rU;
                $var(orig_rdomain) = $rd;
            
                if ($var(fork_to_myself) && $hdr(Content-Type) == "application/im-iscomposing+xml") {
                    $var(no_fork_reason) = 'iscomposing';
                    $var(fork_to_myself) = 0;
                }
                if ($var(fork_to_myself) && search_body("message/imdn")) {
                    $var(no_fork_reason) = 'imdn';
                    $var(fork_to_myself) = 0;
                }

                if ($var(fork_to_myself) && $var(orig_ru) == 'sip:$fU@$fd') {
                    $var(no_fork_reason) = 'myself';
                    $var(fork_to_myself) = 0;
                }
            
                if ($var(fork_to_myself) && is_present_hf("X-Replicated-Message")) {
                    $var(no_fork_reason) = 'x-replicated';
                    $var(fork_to_myself) = 0;
                } 

                if ($hdr(Content-Type) == "application/sylk-api-token") {
                    $var(no_fork_reason) = 'token';
                    $var(fork_to_myself) = 0;
                }

                if ($hdr(Content-Type) == "application/sylk-api-pgp-key-lookup") {
                    $var(no_fork_reason) = 'keylookup';
                    $var(fork_to_myself) = 0;
                }
                
                if ($hdr(Content-Type) == "text/pgp-private-key") {
                    $var(no_fork_reason) = 'privatekey';
                    $var(fork_to_myself) = 0;
                }

                if ($hdr(Content-Type) == "text/pgp-public-key") {
                    $var(no_fork_reason) = 'publickey';
                    if (is_present_hf("X-Sylk-To-Sip")) {
                        $avp(sylk_to_sip) := 1;
                        remove_hf("X-Sylk-To-Sip");
                        $var(fork_to_myself) = 0;
                    }
                }
                
            }

            t_on_branch("MESSAGE_BRANCH");

            $var(orig_ru) = $ru;

            if ($var(fork_to_myself) == 0) {
                DEBUG("Message fork to myself skipped because $var(no_fork_reason) ($ci)");
            }

            sinclude(PREPROCESS_MESSAGE_M4)dnl;

M4_IF(ENABLE_SYLK_MESSAGE_STORAGE);
            if ($avp(source_ip) != "SYLK_SERVER_IP" and $avp(source_port) != "SYLK_SERVER_PORT") {
                # Inform Sylk to replicate an outgoing message
                append_hf("X-Sylk-From-Sip: yes\r\n");
            }
M4_ENDIF(ENABLE_SYLK_MESSAGE_STORAGE);
            if (is_uri_host_local()) {
                if (db_does_uri_exist($ru, "subscriber")) {
                
M4_IF(ENABLE_SYLK_MESSAGE_STORAGE);
                    
                    if ($hdr(Content-Type) == "application/sylk-api-token" || $hdr(Content-Type) == "application/sylk-api-pgp-key-lookup") {
                        # api-token requests from ourselves must be only sent to Sylk Server
                        if ($avp(source_ip) != "SYLK_SERVER_IP" && $avp(source_port) != "SYLK_SERVER_PORT") {
                            INFO("Route $rm $hdr(Content-Type) $ru to Sylk Server SYLK_SERVER_IP");
                            append_hf("X-Sylk-App: webrtcgateway\r\n");
                            $du = "sip:SYLK_SERVER_IP";
                            $dp = "SYLK_SERVER_PORT";
                            force_send_socket("SYLK_SERVER_SOCKET");
                            if (not t_relay()) {
                                sl_reply_error();
                            }
                            exit;
                        } 
                        # replies from Sylk Server with token will be routed only to the end-points
                    } else if (is_present_hf("X-Replicated-Message")) {
                        #INFO("Skip forking $hdr(Content-Type) X-Replicated-Message MESSAGE to Sylk server ($ci)");
                        #exit;
                    }
                    
M4_ENDIF(ENABLE_SYLK_MESSAGE_STORAGE);
                
                    if (lookup("LOCATION_TABLE")) {
M4_IF(ENABLE_SYLK_MESSAGE_STORAGE);
                        if (!is_present_hf("X-Replicated-Message")) {
                            append_branch();
                            INFO("Fork $rm $hdr(Content-Type) $ru to Sylk Server SYLK_SERVER_IP:SYLK_SERVER_PORT");
                            seturi("sip:SYLK_SERVER_IP:SYLK_SERVER_PORT;transport=SYLK_SERVER_TRANSPORT");
                            setbflag("webrtcgateway");
                        }
M4_ENDIF(ENABLE_SYLK_MESSAGE_STORAGE);

                        if ($var(fork_to_myself)) {
                            $var(fu) = "sip:" + $fU + "@" + $fd;
                            INFO("Fork $rm $hdr(Content-Type) for $var(orig_ru) to myself $var(fu) ($ci)");
                            append_branch();
                            seturi($var(fu));
                            setbflag("x_replicated_message");
                        }
 
                        if (not t_relay()) {
                            sl_reply_error();
                        }
                        exit;
                    } else {
M4_IF(ENABLE_SYLK_MESSAGE_STORAGE);
                        INFO("$ru is not yet online ($ci)");
                        if (!is_present_hf("X-Replicated-Message")) {
                            INFO("Forward $rm $hdr(Content-Type) for $var(orig_ru) to Sylk Server SYLK_SERVER_IP");
                            $ru = "sip:SYLK_SERVER_IP:SYLK_SERVER_PORT";
                            setbflag("webrtcgateway");
                        }

                        if ($var(fork_to_myself)) {
                            $var(fu) = "sip:" + $fU + "@" + $fd;
                            INFO("Fork $rm $hdr(Content-Type) for $var(orig_ru) to myself $var(fu) ($ci)");
                            append_branch();
                            seturi($var(fu));
                            setbflag("x_replicated_message");
                        }
M4_ELSE(ENABLE_SYLK_MESSAGE_STORAGE);
                        sl_send_reply(480, "User not online");
                        return;
M4_ENDIF(ENABLE_SYLK_MESSAGE_STORAGE);

                    }
                } else {
                    sinclude(HANDLE_USER_NOT_FOUND_M4)dnl;
                    sl_send_reply(404, "User not found");
                    return;
                }
            } else {
                # uri is not local
M4_IF(ENABLE_SYLK_MESSAGE_STORAGE);
                INFO("Fork $rm $hdr(Content-Type) for $ru to Sylk Server SYLK_SERVER_IP:SYLK_SERVER_PORT");
                append_branch();
                seturi("sip:SYLK_SERVER_IP:SYLK_SERVER_PORT;transport=SYLK_SERVER_TRANSPORT");
                append_hf("X-Sylk-App: webrtcgateway\r\n");
                setbflag("webrtcgateway");
                if ($var(fork_to_myself)) {
                    $var(fu) = "sip:" + $fU + "@" + $fd;
                    INFO("Fork $rm $hdr(Content-Type) for $var(orig_ru) to myself $var(fu) ($ci)");
                    append_branch();
                    seturi($var(fu));
                    setbflag("x_replicated_message");
                 }
M4_ENDIF(ENABLE_SYLK_MESSAGE_STORAGE);
                 
            }
        } else {
            # In-dialog MESSAGE
            if (not loose_route()) {
                # Only relay in-dialog requests that were previously Record-Routed by us
                sl_send_reply(400, "In-dialog MESSAGE rejected");
                exit;

            }
        }

    } else if (IS_REFER) {
        $avp(source_ip) = $si;
        $avp(source_port) = $sp;
        $avp(sip_proxy_ip) = "SERVER_IP";

        if (OUT_OF_DIALOG) {
            if (loose_route()) {
                WARN("Incorrectly formatted $rm request. Rejected with 400. ($ci)");
                sl_send_reply(400, "Incorrectly formatted request");
                return;
            }

            if (FROM_SELF) {
                sl_send_reply(404, "Domain not served here");
                return;
            }

            if (is_from_local()) {
                if (DO_AUTHENTICATE) {
                    if (not proxy_authorize("", "SUBSCRIBER_TABLE")) {
                        DEBUG("Asking credentials (reason=$retcode)");
                        proxy_challenge("");
                        return;
                    } else if ($au != $fU) {
                        sl_send_reply(403, "Username!=From not allowed ($au!=$fU)");
                        return;
                    }

                    if ($rd == "") {
                        $rd = $fd;
                    }

                    # Hide auth credentials to downstream routers
                    consume_credentials();
                }
            }

            if (is_uri_host_local()) {
                if (db_does_uri_exist($ru, "subscriber")) {
                    if (lookup("LOCATION_TABLE")) {
                        t_on_branch("SETUP_RELAY");
                    } else {
                        sl_send_reply(480, "User not online");
                        return;
                    }
                } else{
                    sinclude(HANDLE_USER_NOT_FOUND_M4)dnl;
                    sl_send_reply(404, "User not found");
                    return;
                }
            }
        } else {
            # In-dialog REFER
            if (not loose_route()) {
                # Only relay in-dialog requests that were previously Record-Routed by us
                sl_send_reply(400, "Incorrectly formatted request");
                exit;
            }
        }

        t_on_reply("DIALOG_REPLY");
        t_on_failure("FAILED_DIALOG");
    } else if (CALL_METHODS or IS_OPTIONS) {
        # re-INVITEs, BYE, CANCEL, ACK, PRACK, UPDATE and OPTIONS are handled here

        # Check if R-URI is a GRUU
        if ($(ru{uri.params}) != "" and $(ru{uri.params}{param.exist,gr}) == 1) {
            $var(uri_is_gruu) = 1;
            if (lookup("LOCATION_TABLE")) {
                if ($du) {
                    $var(tmp_ruri_user) = $rU;
                    $var(target) = $(du{uri.param,target}{s.unescape.param});
                    if ($var(target)) {
                        $ru = $var(target);
                    } else {
                        $ru = $du;
                    }
                    $rU = $var(tmp_ruri_user);
                }
            }
        } else {
            $var(uri_is_gruu) = 0;
        }

        if (loose_route()) {
            DEBUG("Loose route for $rm");
            if (IS_ACK and t_check_trans()) {
                t_relay();
                return;
            } else if (is_uri_host_local() and not $var(uri_is_gruu)) {
                WARN("Rejected with 482 badly formated $rm request that will loop back ($ci)");
                if (IS_NOT_ACK) {
                    sl_send_reply(482, "Loop detected");
                }
                return;
            }
        } else {
            if (IS_CANCEL and not t_check_trans()) {
                WARN("Received CANCEL that does not belong to a transaction!");
                # do not give any reply, so that the UA will retransmit the
                # CANCEL until it finds the transaction
                return;
            } else if (IS_ACK and not t_check_trans()) {
                WARN("Received ACK that does not belong to a transaction!");
                return;
            } else if (IS_NOT_CANCEL and IS_NOT_ACK and is_uri_host_local()) {
                DEBUG("Rejected badly formated $rm request that will loop back.");
                sl_send_reply(482, "Loop detected");
                return;
            }
        }

        if ($var(uri_is_gruu) and FROM_SELF and is_present_hf("Self-Forwarded")) {
            # This is a self-forwarded in-dialog request coming from the local_route
            # we may relay it now -Saul
            remove_hf("Self-Forwarded");
            t_relay();
            exit;
        }

        if (IS_INVITE) {
            t_on_reply("INVITE_REPLY");
            t_on_failure("FAILED_REINVITE");
        } else if (IS_UPDATE) {
            t_on_reply("DIALOG_REPLY");
        } else if (IS_NOT_ACK) {
            t_on_reply("OTHER_REPLY");
        }

    } else {
        sl_send_reply(406, "Method not acceptable");
        return;
    }

    route(SEND_OUT);
}



route[REGISTER] {
    if (is_from_local()) {
        $avp(source_ip) = $si;
        $avp(source_port) = $sp;

        if (client_nat_test(3)) {
            if ($socket_in(proto) == "udp") {
                nat_keepalive();
            } else {
                # Keep TCP/TLS connections open until the registration expires
                setflag("PERSISTENT_TCP_FLAG");
                # Do not attempt to initiate an outbound connection, reuse one
                setbflag("TCP_NO_NEW_CONN_BFLAG");
            }
            force_rport();
            $avp(received_uri) = $source_uri;
        }

M4_IF(ENABLE_AUTHENTICATION)
        if (not www_authorize("", "SUBSCRIBER_TABLE")) {
            RDEBUG("Asking credentials (reason=$retcode)");
            www_challenge("");
            return;
        } else if ($au != $tU) {
            RWARN("Username!=To not allowed ($au!=$tU)");
            sl_send_reply(403, "Username!=To not allowed ($au!=$tU)");
            return;
        }
M4_ENDIF(ENABLE_AUTHENTICATION)
        if (not save("LOCATION_TABLE", "f")) {
            RERROR("Failed to save user location");
            sl_reply_error();
        } else {
            RINFO("REGISTER contact $hdr(Expires) sec for $tU@$td using $hdr(User-Agent): $hdr(Contact)");
            sinclude(POSTPROCESS_REGISTER_M4)dnl;
        }

    } else {
        RWARN("Send 403: Domain $fd is not served here: $fu");
        sl_send_reply(403, "This domain is not served here");
    }
}

#
route[LOOKUP_DESTINATION] {
    INFO("Lookup destination for $ru ($ci)"); 

    $avp(can_uri) = $ru;
    
    if ($rd =~ "CONFERENCE_SERVER_PREFIX.*") {
         $avp(tmp) := $rd;
         avp_subst("$avp(tmp)", "/^CONFERENCE_SERVER_PREFIX\.//i");
         if (is_domain_local($avp(tmp))) {
             INFO("Forwarding conference server request to Sylk Server at SYLK_SERVER_IP:SYLK_SERVER_PORT");
             $ru = "sip:SYLK_SERVER_IP:SYLK_SERVER_PORT";
             force_send_socket("SYLK_SERVER_SOCKET");
         }
    }

    if (is_uri_host_local()) {
        # Handle local public destinations
        if ($ru =~ "^sip:TEST_NUMBERS@.*") {
             INFO("Forwarding $rm to Sylk Server at SYLK_SERVER_IP:SYLK_SERVER_PORT");
             $ru = "sip:" + $rU + "@" + "SYLK_SERVER_IP:SYLK_SERVER_PORT";
             force_send_socket("SYLK_SERVER_SOCKET");
             
M4_IF(ENABLE_MEDIAPROXY)
            if (not FROM_SELF) {
               INFO("Engaging mediaproxy for call $ci"); 
               engage_media_proxy();
            }
M4_ENDIF(ENABLE_MEDIAPROXY)
             t_on_reply("INVITE_REPLY");
             t_on_failure("FAILED_TRANSIT");

             route(SEND_OUT);
             exit;
        } 
    
        if (lookup("LOCATION_TABLE")) {
            INFO("$rU@$rd has online devices for $rm ($ci)");
M4_IF(ENABLE_MEDIAPROXY)
            if (not FROM_SELF) {
               INFO("Engaging mediaproxy for call $ci"); 
               engage_media_proxy();
            }
M4_ENDIF(ENABLE_MEDIAPROXY)
        } else {
            sl_send_reply(480, "User not online");
            exit;
        }
    } else {
        # URI host is not local (call to non local subscriber)
M4_IF(ENABLE_MEDIAPROXY)
        if (not FROM_SELF) {
           INFO("Engaging mediaproxy for call $ci"); 
           engage_media_proxy();
        }
M4_ENDIF(ENABLE_MEDIAPROXY)
        DEBUG("Set timeout to OUTGOING_TIMEOUT");
        $T_fr_inv_timeout = OUTGOING_TIMEOUT;

        t_on_reply("INVITE_REPLY");
        t_on_failure("FAILED_TRANSIT");

        route(SEND_OUT);
        exit;
    }
}

route[SEND_OUT] {
    INFO("Send out route for $ru ($ci)");
    sinclude(POSTPROCESS_REQUEST_M4)dnl;

    if (IS_FIRST_INVITE and is_domain_local($fd)) {
        if (FROM_SELF) {
            $avp(call_token) := $hdr(Call-Token);
        } else {
            $avp(call_token) := $RANDOM;
        }
    }

    record_route();

    if (IS_CANCEL) {
        if (not t_relay(0x08)) {
            INFO("Internal server error for $rm to $ru ($ci)");  
            sl_reply_error();
        }
    } else {
        if (CALL_METHODS) {
            INFO("Sending out $rm request to $ru ($ci)");  
        } else if (IS_MESSAGE){
            INFO("Sending out $rm request to $ru ($ci)");  
        } else {
            DEBUG("Sending out $rm request to $ru ($ci)");  
        }
        
        t_relay(0x02);
        $var(retcode) = $retcode;
        if (IS_NOT_ACK) {
            if (IS_INVITE and $var(retcode) < 0) {
                INFO("Internal server error 500 for CALL to $ru ($ci)");  
            }
            if ($var(retcode) == -6) {
                if (CALL_METHODS) {
                    INFO("Internal server error 503 for $rm to $ru ($ci)");  
                } else if (IS_MESSAGE){
                    INFO("Internal server error 503 for $rm to $ru ($ci)");  
                } else {
                    DEBUG("Internal server error 503 for $rm to $ru ($ci)");  
                }
                send_reply(503, "Service Unavailable");
            } else if ($var(retcode) < 0) {
                if (CALL_METHODS) {
                    INFO("Internal server error for $rm to $ru ($ci)");  
                } else if (IS_MESSAGE){
                    INFO("Internal server error for $rm to $ru ($ci)");  
                } else {
                    DEBUG("Internal server error for $rm to $ru ($ci)");  
                }
                sl_reply_error();
            }
        }
        #exit;
    }
}


#
# The branch routes
#

branch_route[SETUP_RELAY] {
    INFO("Setup relay for $rm CALL ($ci): $rs $rr");
    sinclude(PREPROCESS_BRANCH_M4)dnl;
}


branch_route[MESSAGE_BRANCH] {
    INFO("Dialog branch for $rm ($ci): $ru");

    if (isbflagset("x_replicated_message")) {
        DEBUG("Append X-Replicated-Message");
        append_hf("X-Replicated-Message: yes\r\n");
    }

    if (isbflagset("webrtcgateway")) {
        DEBUG("Append X-Sylk-App");
        append_hf("X-Sylk-App: webrtcgateway\r\n");
    }

    if ($avp(sylk_to_sip) == 1) {
        exit;
    }
        
}

#
# The reply routes
#

onreply_route[INVITE_REPLY] {
    if (not FROM_SELF) {
        INFO("Reply for $rm CALL ($ci): $rs $rr");
        if ($hdr(Contact) and $hdr(Record-Route)) {
            # when listening on multiple interfaces compare with $var(via_host) instead
            #$var(via_host) = $(hdr(Via){via.host});
            $var(rr_host) = $(hdr(Record-Route){s.select,0,,}{nameaddr.uri}{uri.host});
            if (is_myself($var(rr_host)) and client_nat_test(1)) {
                fix_contact();
            }
        }
    } else {
        INFO("Reply for self-forwarded $rm CALL ($ci): $rs $rr");
    }
}



onreply_route[DIALOG_REPLY] {
    if (CALL_METHODS) {
        INFO("Reply for $rm CALL ($ci): $rs $rr");
    } else {
        INFO("Reply for $rm from $si ($ci): $rs $rr");
    }

    if ($hdr(Contact) and $hdr(Record-Route)) {
        $var(rr_host) = $(hdr(Record-Route){s.select,0,,}{nameaddr.uri}{uri.host});
        if (is_myself($var(rr_host)) and client_nat_test(1)) {
            fix_contact();
        }
    }
}


onreply_route[OTHER_REPLY] {
    # Reply route used by Bye, Cancel, Update and Prack requests

    if (not FROM_SELF) {
        INFO("Reply for $rm ($ci): $rs $rr");
    } else {
        INFO("Reply for self-forwarded $rm ($ci): $rs $rr");
    }
}



#
# The failure routes
#

failure_route[FAILED_REINVITE] {
    INFO("In dialog CALL re-INVITE failed with $(<reply>rs) $(<reply>rr) ($ci)");
}


failure_route[FAILED_TRANSIT] {
    if (t_check_status("407")) {
        INFO("Remote proxy does not allow calls from outside");
        t_reply(403, "Remote proxy does not allow calls from outside");
    } else {
        INFO("Call failed with $(<reply>rs) $(<reply>rr) ($ci)");
    }
}

failure_route[FAILED_CALL] {
    if (t_check_status("407")) {
        INFO("Remote proxy does not allow calls from outside");
        t_reply(403, "Remote proxy does not allow calls from outside");
    } else {
        INFO("Call failed with $(<reply>rs) $(<reply>rr) ($ci)");
    }
}


failure_route[FAILED_DIALOG] {
    if (CALL_METHODS) {
        INFO("In dialog CALL $rm failed with $(<reply>rs) $(<reply>rr) ($ci)");
    } else {
        INFO("In dialog $rm failed with $(<reply>rs) $(<reply>rr) ($ci)");
    }
}


failure_route[FAILED_MESSAGE] {
    INFO("Message failed with $(<reply>rs) $(<reply>rr) at $si:$sp ($ci)");
}


local_route {
    INFO("$rm (generated) from $fu to $tu via $ru ($ci)");
}
