
#
# OpenSIPS settings and parameters to customize the installation.
#

################
# Basic settings
################

# 0 is unsafe but a good start to see your proxy working
define(`ENABLE_AUTHENTICATION', `0')

# relay RTP packets using mediaproxy (necessary to receive calls behind NAT)
define(`ENABLE_MEDIAPROXY', `0')

# MySQL parameters
define(`MySQL_URI', `mysql://opensips:opensips@localhost/opensips')

# TLS advanced parameters
define(`TLS_VERIFY_CERTIFICATE', `0')

# Sylk Server settings
define(`TEST_NUMBERS', `(echo|playback)')
define(`ENABLE_SYLK_MESSAGE_STORAGE', `1')

define(`CONFERENCE_SERVER_PREFIX', `conference')
define(`SYLK_SERVER_IP', `127.0.0.1')
define(`SYLK_SERVER_PORT', `5060')
define(`SYLK_SERVER_TRANSPORT', `tcp')
define(`SYLK_SERVER_SOCKET', `SERVER_TCP_SOCKET')

###################
# Advanced settings
###################

# The IP on which OpenSIPS will listen (by default the IP for the default interface)
define(`SERVER_IP', esyscmd(`echo -n $(hostname -i)'))

define(`SERVER_UDP_PORT', `15060')
define(`SERVER_TCP_PORT', `15060')
define(`SERVER_TLS_PORT', `15061')

# SERVER aliases
# Example:
#  define(`SERVER_ALIAS1', `example.com')
#  define(`SERVER_ALIAS2', `example.net')
#  define(`SERVER_ALIASES', `2')
define(`SERVER_ALIASES', `0')

# How many worker processes to spawn (will start with min but will grow up to
# max as load increases; will also scale down back to min if load decreases)
# TCP and UDP worker pools are separate and vary independently
define(`MIN_WORKERS', `5')
define(`MAX_WORKERS', `10')

define(`MySQL_URI_siptrace', `MySQL_URI')
define(`MySQL_URI_location', `MySQL_URI')
define(`MySQL_URI_domain',   `MySQL_URI')

# Set this to 1 for encrypted passwords or 0 for plaintext passwords
# Boolean value (0 or 1 only)
define(`ENCRYPTED_PASSWORDS', `1')

# The log levels defined below can have one of these values:
#
#   -1: ERROR, 1: WARNING, 2: NOTICE, 3: INFO, 4: DEBUG
#
# The log levels can be adjusted at runtime using the MI interface.

# The log level for the opensips processes
define(`PROC_LOG_LEVEL', `-1')

# The log level for the configuration script
define(`CONF_LOG_LEVEL', `3')

# Enable configuration logging for REGISTER. Boolean value (0 or 1 only)
define(`REGISTER_LOGGING', `1')

# How many open files can opensips process (this affects how many TCP/TLS
# connections can be accepted). Increasing this increases memory usage and
# reduces performance. Valid values are in the 1000-1000000 range.
define(`LIMIT_FILENO', `1000')

# MySQL advanced parameters
define(`DOMAIN_TABLE',     `domain')
define(`ALIASES_TABLE',    `dbaliases')
define(`LOCATION_TABLE',   `location')
define(`SUBSCRIBER_TABLE', `subscriber')

# Timeouts (INVITE/other methods)
define(`INVITE_TIMEOUT',   `35')
define(`OTHERS_TIMEOUT',   `15')
define(`OUTGOING_TIMEOUT', `120')

# Registration expiration restrictions
# DEFAULT_EXPIRE is for clients that do not set an expiration time
# MIN and MAX_EXPIRE will force expiration times to be contained in that
# interval, no matter what the client proposes.
define(`MIN_EXPIRE',     `60')
define(`MAX_EXPIRE',     `3600')
define(`DEFAULT_EXPIRE', `300')

# Interval to ping the clients behind NAT to keep the pinholes open
define(`NAT_KEEPALIVE_INTERVAL', `80')

# QoS control. The type of service (TOS) value can be used to control the
# traffic priority at the IP level, however it has to be supported by the
# switches and routers in the path to actually work. The value is either a
# hex value written like 0x10, or one of the following keywords:
#   IPTOS_LOWDELAY, IPTOS_THROUGHPUT, IPTOS_RELIABILITY or IPTOS_MINCOST
define(`TOS', `IPTOS_LOWDELAY')


# MediaProxy ICE candidate (none | low-priority | high-priority)
define(`MEDIAPROXY_ICE_CANDIDATE', `none')

# enable tracing to sip_trace MySQL tables
define(`TRACING', `0')
